=begin
(c) TIG 2012
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  PCFileTools.rb

Adds new PC ONLY tools similar to the Ruby 'File' and 'Dir' tools, 
BUT which avoid issues with UFT-8 characters in filepaths etc...
[NOTE: MACs have no equivalent issues so they don't need this!] 

  PCFile... & PCDir...
  
They generally work like the Ruby equivalent
So
  File.exist?("C:/Temp/bou.txt")
which will WRONGLY return 'false,' is replaced by
  PCFile.exist?("C:/Temp/bou.txt")
which correctly returns 'true'...

###

PCFile.basename( fileName [, aSuffix] )
  returns the basename of the file or folder, e.g. "C:/Temp/file.txt" 
  -> "file.txt" ; if the 'aSuffix' is used then any matching extension 
  is stripped, using ".*" removes ANY file extension - e.g. -> "file".
  
PCFile.delete( fileName )
  deletes the given file; if it's not an 'absolute path' the file is 
  sought in the pwd.
  
PCFile.dirname( fileName)
  returns the directory [path] of the passed filepath, stripping the 
  terminal file/folder; e.g. "C:/Temp/file.txt" -> "C:/Temp"
  
PCFile.directory?( fileName )
  returns true or false, depending on whether or not the given 
  'fileName' refers to a 'directory'.
  
PCFile.exist?( fileName )
  returns true or false, depending on whether or not the given file 
  [OR folder] exists.
  
PCFile.extname( fileName )
  returns the extension-name [if any] of the passed file; 
  e.g. "C:/Temp/file.txt" > ".txt"
  
PCFile.ftype( fileName )
  returns the type of the object as 'directory' or 'file'.
  [Note that it is more limited that native File.ftype()]
  
PCFile.join( string, string* )
  joins strings to return a file-path; 
  e.g. PCFile.join("C:/Temp", "bou.txt") -> "C:/Temp/bou.txt"
  
PCFile.rename( fileName, newName )
  renames the file 'fileName' as 'newName'; unlike File.rename(...) it 
  can 'move' a file to a new folder, if the specified folder exists, 
  if no folder path is given it assumes the same folder as 'fileName'.
  
PCFile.move( fileName, nname )
  equivalent to PCFilename.rename(...)
  
PCFile.copy( fileName, newName, [overwrite=false] )
  makes a copy of the file 'fileName' as 'newName'; it can 'copy' a file 
  to a new folder, if the specified folder exists, if no folder path is 
  given it assumes the same folder as 'fileName'.
  If a 'true' 3rd  argument is given then any existing file is 
  overwritten, if a 'false' or no 3rd argument is given then any 
  existing file is NOT overwritten.
  
PCFile.new( fileName, [modeString="r"] )
  opens a file as f=PCFile.new(...), making it or using it if it exists, 
  the mode defaults to "r" = 'read', the options are:
  "r"  =read
  "r+" =read+write
  "w"  =write
  "w+" =write+read
  "a"  =append
  "a+" =append+read
  
PCFile.open( fileName, [modeString="r"] )
  equivalent to PCFile.new(...)
  
NOTE: the slightly different syntax for the read/write/close file 'f'...
[there is no 'binmode']

PCFile.read(f)
  reads all of the contents of the new/open file 'f'.
  
PCFile.readline(f)
  reads the first line of the new/open file 'f'.
  
PCFile.readlines(f)
  reads all of the lines of the new/open file 'f' as an array of lines.
  
PCFile.write(f, string)
  writes the 'string' to the new/open file 'f', with no added \n
  
PCFile.print(f, string)
  writes the 'string' to the new/open file 'f', with no added \n
  
PCFile.puts(f, string|array)
  writes the 'string' to the new/open file 'f', adding a \n;
  if it's an array it writes it a line at a time adding \n to each line.
  
PCFile.close(f)
  closes the new/open file 'f' made as above, after you've finished 
  reading/writing it etc.
  
###

PCDir.pwd
  returns the path to the current directory [folder].
  
PCDir.chdir( dirPath )
  changes the current directory [folder] to 'dirPath', if it exists.
  
PCDir.mkdir( dirPath )
  makes the specified new directory [folder].
  
PCDir.rmdir( dirPath )
  deletes the specified directory - note that unlike the Dir. method it 
  will remove a directory [folder] even if it contains files!
  
PCDir.exist?( dirPath )
  returns true or false, depending on whether or not the folder exists.
  
PCDir.copy( dirPath, dirNew )
  makes a copy of a folder at the new location/name.
  
PCDir.move( dirPath, dirNew )
  moves a folder to the new location/name.
  
PCDir.entries( dirPath )
  returns an array of the names all of the files/folders within the 
  specified folder [including '.' and '..']
  
NOTE:
Requires a compatible version of 'win32ole.so' in the Plugins folder.
Available from here:
http://forums.sketchucation.com/viewtopic.php?p=380121#p380121

Donations:
info @ revitrev.org via PayPal.com

Version:
1.0 20120202 First release.
###
=end

###
require 'sketchup.rb'
###

#################################

###
module PCDir
###
unless [RUBY_PLATFORM].grep(/mswin/)==[RUBY_PLATFORM]
  UI.messagebox("PCFile: is a PC only toolset!")
end
require 'win32ole.so'

@fso = WIN32OLE.new("Scripting.FileSystemObject")

def self.pwd()
  return @fso.GetAbsolutePathName(".")
end#

def self.chdir(dname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  return false unless self.exist?(dname)
  oShell = WIN32OLE.new("WScript.Shell")
  cd = oShell.CurrentDirectory = dname.tr("\\","/")
  WIN32OLE.ole_free(oShell)
  oShell=nil
  GC.start
  return cd
end#

def self.mkdir(dname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  return @fso.CreateFolder(dname).Path.tr("\\","/")
end#

def self.rmdir(dname)### erases ALL contents!
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  return @fso.DeleteFolder(dname) 
end#

def self.entries(dname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  dir=@fso.GetFolder(dname)
  subfs=dir.SubFolders
  files=dir.Files
  names=[".", ".."]
  subfs.each{|e|names << e.Name}
  files.each{|e|names << e.Name}
  names.sort
end#

def self.copy(dname, nname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  nin=nname
  begin
    nname=nname.unpack("U*").map{|c|c.chr}.join
  rescue
    nname=nin
  end
  return @fso.CopyFolder(dname, nname)
end#

def self.move(dname, nname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  nin=nname
  begin
    nname=nname.unpack("U*").map{|c|c.chr}.join
  rescue
    nname=nin
  end
  return @fso.MoveFolder(dname, nname)
end#

def self.exist?(dname)
  din=dname
  begin
    dname=dname.unpack("U*").map{|c|c.chr}.join
  rescue
    dname=din
  end
  return @fso.FolderExists(dname)
end#

###
end#module
###

#########################

###
module PCFile
###

unless [RUBY_PLATFORM].grep(/mswin/)==[RUBY_PLATFORM]
  UI.messagebox("PCFile: is a PC only toolset!")
end
require 'win32ole.so'

@fso = WIN32OLE.new("Scripting.FileSystemObject")

def self.exist?(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  if @fso.FileExists(fname)
    return true
  else
    if self.directory?(fname)
	  return true
	else
      return false
	end
  end
end#

def self.directory?(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  if @fso.FolderExists(fname)
    return true
  else
    return false
  end
end#

def self.dirname(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  return @fso.GetParentFolderName(fname)
end#

def self.basename(fname, ext=nil)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  if ext and ext!=""
    ein=ext
	begin
	  ext=ext.unpack("U*").map{|c|c.chr}.join
	rescue
	  ext=ein
	end
    base=@fso.GetFileName(fname)
	bext=self.extname(fname)
    if ext==".*"
	  base.gsub!(/#{bext}$/, "")
	else ### matching name
	  dext=bext.downcase
	  base.gsub!(/#{bext}$/, dext)
	  ext=ext.downcase
	  base.gsub!(/#{ext}$/, "")
	  base.gsub!(/#{dext}$/, bext) ### IF unchnaged
	end#ext
    return base
  else
    return @fso.GetFileName(fname)
  end
end#

def self.extname(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  return "."+@fso.GetExtensionName(fname)  
end#

def self.join(*args)
  str=""+args[0].to_s
  args[1..-1].each{|e|
    str << "/"+e.to_s
  }
  return str.tr("\\", "/")
end#

def self.delete(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  if self.directory?(fname)
    return false
  elsif self.exist?(fname)
    @fso.DeleteFile(fname)
	return true
  else
    return false
  end
end#

def self.ftype(fname)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  if self.exist?(fname)
    if self.directory?(fname)
	  return "directory"
	else
	  return "file"
	end
  else
    return false
  end
end#

def self.new(fname, mode)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  case mode.downcase
    when "r"
	  mod=1
    when "r+"
	  mod=3
	when "w"
	  mod=2
	when "w+"
	  mod=3
	when "a"
	  mod=8
	when "a+"
	  mod=9
	else ### default=read
	  mod=1
  end
  return @fso.OpenTextFile(fname, mod, true, -2)
end#
def self.open(fname, mode)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  self.new(fname, mode)
end#
def self.close(file=nil)
  return nil unless file
  file.Close
  WIN32OLE.ole_free(file)
  file=nil
  GC.start
end

def self.rename(fname=nil, nname=nil)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  nin=nname
  begin
    nname=nname.unpack("U*").map{|c|c.chr}.join
  rescue
    nname=nin
  end
  if not fname or not nname or fname.empty? or nname.empty?
    return false
  elsif not self.exist?(fname)
    return false
  else
    if self.directory?(fname) or self.directory?(nname)
	  return false
	else
	  fname.tr!("\\","/")
	  nname.tr!("\\","/")
	  nname.gsub!(/^\//,"")
	  ### ensure nname is a full path OR if not use fname's
	  ### unlike File.rename if will move to a new folder!
	  if nname=~/[\/]/ ### NOT a simple file!
	    if nname=~/[A-Za-z][:]/ and not self.exist?(self.dirname(nname))
		  return false ### folder missing
		elsif not self.exist?(self.dirname(self.join(self.dirname(fname), nname))) 
	      return false ### folder missing
	    end
	    nname=self.join(self.dirname(fname), nname)
	  else
	    nname=self.join(self.dirname(fname), nname)
	  end
      @fso.MoveFile(fname, nname)
	  return true
	end
  end
end#
def self.move(fname=nil, nname=nil)
  self.rename(fname, nname)
end#
def self.copy(fname=nil, nname=nil, overwrite=false)
  fin=fname
  begin
    fname=fname.unpack("U*").map{|c|c.chr}.join
  rescue
    fname=fin
  end
  nin=nname
  begin
    nname=nname.unpack("U*").map{|c|c.chr}.join
  rescue
    nname=nin
  end
  if overwrite and self.exist?(nname)
    return false
  end
  if fname.empty? or nname.empty?
    return false
  elsif not self.exist?(fname)
    return false
  else
    if self.directory?(fname) or self.directory?(nname)
	  return false
	else
	  fname.tr!("\\","/")
	  nname.tr!("\\","/")
	  nname.gsub!(/^\//,"")
	  ### ensure nname is a full path OR if not use fname's
	  ### unlike File.rename if will move to a new folder!
	  if nname=~/[\/]/ ### NOT a simple file!
	    if nname=~/[A-Za-z][:]/ and not self.exist?(self.dirname(nname))
		  return false ### folder missing
		elsif not self.exist?(self.dirname(self.join(self.dirname(fname), nname))) 
	      return false ### folder missing
	    end
	    nname=self.join(self.dirname(fname), nname)
	  else
	    nname=self.join(self.dirname(fname), nname)
	  end
      @fso.CopyFile(fname, nname)
	  return true
	end
  end
end#

def self.read(file=nil)
  return nil unless file
  file.ReadAll(str)
end#
def self.readlines(file=nil)
  return nil unless file
  file.ReadAll(str).split("\n")
end#
def self.readline(file=nil)
  return nil unless file
  file.ReadLine(str)
end#
def self.write(file=nil, str="")
  return nil unless file
  file.Write(str)
end#
def self.print(file=nil, str="")
  self.write(file, str)
end#
def self.puts(file=nil, str="")
  return nil unless file
  if str.class==String
    file.WriteLine(str.chomp!)
  elsif tr.class==Array
    str.each{|st|file.WriteLine(st.chomp!)}
  end
end#

###
end#module
###